//
// This demo shows how to load unclean data from a csv file and preprocess it.
//
// Use Enter to switch between the original and the processed tables.
//
// @author Pierre Dragicevic
//

import java.util.regex.Pattern;
import java.util.regex.Matcher;

Table originalTable, table;
TableViewer originalTableViewer, tableViewer;

void setup() {
  
  // Load data into two identical tables
  table = new Table("TimeUse.csv", ",", null);
  originalTable = new Table(table);

  // Set columns to process
  int countryCol = table.getColumnIndex("GEO/ACL00");
  int firstTimeCol = table.getColumnIndex("Total");
  int lastTimeCol = table.getColumnIndex("Unspecified time use");
  
  // Prepare regular expressions we will need for parsing strings
  // (for more info see, e.g., http://www.vogella.de/articles/JavaRegularExpressions/article.html)
  Pattern hourMinuteRegex = Pattern.compile("([0-9]+):([0-9]+)");
  Pattern parenthesesRegex = Pattern.compile("\\([^)]*\\)");
    
  // In all cells, replace H:M with the total number of minutes
  for (int r = 0; r < table.getRowCount(); r++) {
    for (int c = firstTimeCol; c <= lastTimeCol; c++) {
      String cell = table.get(c, r);
      Matcher matcher = hourMinuteRegex.matcher(cell);
      if (matcher.matches()) {
        int hours = parseInt(matcher.group(1));
        int minutes = parseInt(matcher.group(2));
        table.set(c, r, "" + (hours * 60 + minutes));
      } else {
        table.set(c, r, "0"); // use 0 for unspecified values
      }
    }
  }
  
  // Remove text in parentheses from all country names
  for (int r = 0; r < table.getRowCount(); r++) {
    String cell = table.get(countryCol, r);
    Matcher matcher = parenthesesRegex.matcher(cell);
    String newcell = matcher.replaceAll("");
    table.set(countryCol, r, newcell);
  }
  
  // Show the results: create one viewer for the original table and one for the processed table
  originalTableViewer = new TableViewer(originalTable, true, true);
  tableViewer = new TableViewer(table, true, true);
  tableViewer.colSizes = originalTableViewer.colSizes; // use same column widths
  
  // Uncomment the following line to save the table
  // table.save("TimeUse2.csv", "\t");
}

////////////////////
// Painting code
////////////////////

boolean showOriginalTable = false;

void draw() {
  if (showOriginalTable)
    originalTableViewer.draw();
  else
    tableViewer.draw();
}

void mousePressed() {
  originalTableViewer.mousePressed();
  tableViewer.mousePressed();
}

void mouseDragged() {
  originalTableViewer.mouseDragged();
  tableViewer.mouseDragged();
}

void keyPressed() {
  if (keyCode == ENTER) {
    showOriginalTable = !showOriginalTable; // toggle between tables
    redraw();
  }
}
